; *****************************
; ** angelZware PB Tools Lib **
; ** ----------------------- **
; ** awdate.pbi              **
; ** V0.2 (2010/09/10)       **
; ** by Ronny Krger         **
; *****************************
; ** Licence: LGPL V3        **
; *****************************

;**
;* awdate.pbi_
;* Version 0.2_
;* _
;* Datumsfunktionen fr PureBasic 4.50+_
;* Lizenz: LGPL V3_
;* _
;* Systemanforderungen:_
;* <table style="text-align: left;" border="0" cellpadding="2" cellspacing="0"><tbody><tr>
;* <td style="vertical-align: top;">Windows</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Linux</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Mac</td><td style="vertical-align: top;">[ ]</td></tr><tr>
;* <td style="vertical-align: top;">x86</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">x64</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">PPC</td><td style="vertical-align: top;">[ ]</td></tr><tr>
;* <td style="vertical-align: top;">ASCII</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Unicode</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;"></td><td style="vertical-align: top;"></td></tr></tbody></table>

;{ private functions
Procedure.s ____DATE_GetRFC2822DateTime(date.i)   ; RFC2822-konformen String erzeugen
  Protected rfcdate.s, timezone.s
  
  CompilerSelect #PB_Compiler_OS
    CompilerCase #PB_OS_Windows
      Protected tzh.i, tzm.i, tz.s, tzi.TIME_ZONE_INFORMATION, tzir.i, bias.i
      
      tzir = GetTimeZoneInformation_(tzi.TIME_ZONE_INFORMATION)
      bias = tzi\Bias
      Select tzir
        Case #TIME_ZONE_ID_STANDARD
          If tzi\StandardDate\wMonth
            bias + tzi\StandardBias
          EndIf
        Case #TIME_ZONE_ID_DAYLIGHT
          If tzi\DaylightDate\wMonth
            bias + tzi\DaylightBias
          EndIf
      EndSelect 
      
      date = AddDate(date, #PB_Date_Minute, bias)
      If bias > 0 : timezone = "-" : Else : timezone = "+" : EndIf
      timezone + RSet(Str(Abs(bias / 0.6)), 4, "0")
    CompilerCase #PB_OS_Linux 
      Protected p.i
      
      p = RunProgram("date", "+%z", "", #PB_Program_Open | #PB_Program_Read)
      If p
        timezone = ReadProgramString(p)
        CloseProgram(p)
        date = AddDate(date, #PB_Date_Minute, -(Val(timezone) * 0.6))
      Else ; falls Programm nicht verfgbar ist, oder Aufruf fehlschlgt
        timezone = ""
      EndIf
  CompilerEndSelect
  
  rfcdate  = StringField("Sun|Mon|Tue|Wed|Thu|Fri|Sat", DayOfWeek(date) + 1, "|")
  rfcdate  + ", " + Str(Day(date)) + " "
  rfcdate  + StringField("Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec", Month(date), "|")
  rfcdate  + FormatDate(" %yyyy %hh:%ii:%ss", date)
  If timezone : rfcdate + " " + timezone : EndIf

  ProcedureReturn rfcdate
EndProcedure
Procedure.i ____DATE_ParseRFC2822DateTime(rfc.s)  ; RFC(2)822-konformen String in normales PB-Datumsformat umwandeln
  Protected p.i, tz.s, dat.i, d.s, z.s
  
  p = FindString(rfc, ",", 1)                 ; ggf Tag entfernen
  If p : rfc = Trim(Mid(rfc, p+1)) : EndIf
  
  rfc = ReplaceString(rfc, " Jan ", ".01.")   ; Monatsnamen durch .Ziffer. ersetzen
  rfc = ReplaceString(rfc, " Feb ", ".02.")
  rfc = ReplaceString(rfc, " Mar ", ".03.")
  rfc = ReplaceString(rfc, " Apr ", ".04.")
  rfc = ReplaceString(rfc, " May ", ".05.")
  rfc = ReplaceString(rfc, " Jun ", ".06.")
  rfc = ReplaceString(rfc, " Jul ", ".07.")
  rfc = ReplaceString(rfc, " Aug ", ".08.")
  rfc = ReplaceString(rfc, " Sep ", ".09.")
  rfc = ReplaceString(rfc, " Oct ", ".10.")
  rfc = ReplaceString(rfc, " Nov ", ".11.")
  rfc = ReplaceString(rfc, " Dec ", ".12.")
  d   = StringField(rfc, 1, " ")               ; Datum, Zeit und Zeitzone extrahieren
  z   = StringField(rfc, 2, " ")
  tz  = StringField(rfc, 3, " ")
  
  Select tz                                    ; RFC822-Zeitzonen anpassen               
    Case ""    : tz = "+0000"
    Case "UT"  : tz = "+0000"
    Case "GMT" : tz = "+0000"
    Case "EST" : tz = "-0500"
    Case "EDT" : tz = "-0400"
    Case "CST" : tz = "-0600"
    Case "CDT" : tz = "-0500"
    Case "MST" : tz = "-0700"
    Case "MDT" : tz = "-0600"
    Case "PST" : tz = "-0800"
    Case "PDT" : tz = "-0700"
    Case "A"   : tz = "-0100"
    Case "B"   : tz = "-0200"
    Case "C"   : tz = "-0300"
    Case "D"   : tz = "-0400"
    Case "E"   : tz = "-0500"
    Case "F"   : tz = "-0600"
    Case "G"   : tz = "-0700"
    Case "H"   : tz = "-0800"
    Case "I"   : tz = "-0900"
    Case "K"   : tz = "-1000"
    Case "L"   : tz = "-1100"
    Case "M"   : tz = "-1200"
    Case "N"   : tz = "+0100"
    Case "O"   : tz = "+0200"
    Case "P"   : tz = "+0300"
    Case "Q"   : tz = "+0400"
    Case "R"   : tz = "+0500"
    Case "S"   : tz = "+0600"
    Case "T"   : tz = "+0700"
    Case "U"   : tz = "+0800"
    Case "V"   : tz = "+0900"
    Case "W"   : tz = "+1000"
    Case "X"   : tz = "+1100"
    Case "Y"   : tz = "+1200"
  EndSelect
  
  dat = ParseDate("%dd.%mm.%yyyy %hh:%ii:%ss", d + " " + z)
  dat = AddDate(dat, #PB_Date_Minute, Val(tz) * 0.6)

  ProcedureReturn dat
EndProcedure
;}
;{ public functions
;** awpb_date_datetorfc2822
;* Kovertiert das bergebene Datum/Zeit in einen RFC2822-konformen String.
;** .date.i: Datum, im PureBasic-Format (wie z.B. von Date() geliefert)
Procedure.s AWPB_DATE_DateToRFC2822(date.i)
  ProcedureReturn ____DATE_GetRFC2822DateTime(date)
EndProcedure
;** awpb_date_rfc2822todate
;* Kovertiert das bergebene RFC2822-Datum/Zeit in das PureBasic-Format, welches mit den PB-Datumsfunktionen weiterbearbeitet werden kann.
;** .rfcdate.s: Datum, im RFC2822-Format
Procedure.i AWPB_DATE_RFC2822ToDate(rfcdate.s)
  ProcedureReturn ____DATE_ParseRFC2822DateTime(rfcdate)
EndProcedure
;}
  
; IDE Options = PureBasic 4.51 RC 2 (Linux - x64)
; CursorPosition = 60
; FirstLine = 50
; Folding = D-
; EnableThread
; EnableXP
; CPU = 1
; Compiler = PureBasic 4.51 RC 2 (Windows - x86)