; *****************************
; ** angelZware PB Tools Lib **
; ** ----------------------- **
; ** awfile.pbi              **
; ** V0.2 (2010/08/26)       **
; ** by Ronny Krger         **
; *****************************
; ** Licence: LGPL V3        **
; *****************************

;**
;* awfile.pbi_
;* Version 0.2_
;* _
;* Dateifunktionen fr PureBasic 4.50+_
;* Lizenz: LGPL V3_
;* _
;* Systemanforderungen:_
;* <table style="text-align: left;" border="0" cellpadding="2" cellspacing="0"><tbody><tr>
;* <td style="vertical-align: top;">Windows</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Linux</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Mac</td><td style="vertical-align: top;">[ ]</td></tr><tr>
;* <td style="vertical-align: top;">x86</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">x64</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">PPC</td><td style="vertical-align: top;">[ ]</td></tr><tr>
;* <td style="vertical-align: top;">ASCII</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Unicode</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;"></td><td style="vertical-align: top;"></td></tr></tbody></table>

XIncludeFile "awzip.pbi"

;{ private functions
Procedure.b ____FILE_WriteHeader(fid.i, hdr.s)      ; schreibt einen Header in die Datei (4 Bytes Lnge)
  If fid And hdr
    WriteString(fid, LSet(hdr, 4, " "), #PB_Ascii)
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure
Procedure.b ____FILE_WriteBlock(fid.i, *src, slen.i, flags.b = 0, complevel.i = #____ZIP_COMP_MAX)      ; schreibt einen binren block in die Datei und komprimiert diesen ggf
  Protected *dst, dlen.i, scrc.l, dcrc.l
    
  If fid And *src And slen
    dlen = 0
    *dst = #Null
    scrc = CRC32Fingerprint(*src, slen)   ; Prfsumme der Originaldaten berechnen
    If complevel < #____ZIP_COMP_NONE : complevel = #____ZIP_COMP_NONE : EndIf
    If complevel > #____ZIP_COMP_MAX  : complevel = #____ZIP_COMP_MAX  : EndIf
    If complevel
      *dst = ____ZIP_PackMemory(*src, slen, @dlen, complevel)
    EndIf
    If (dlen = 0) Or (dlen > slen)  ; wenn nicht gepackt oder nach dem packen grer
      WriteLong(fid, 10 + slen)   ; blocksize
      WriteByte(fid, 0)           ; not packed
      WriteByte(fid, flags)       ; #PB_Ascii, #PB_UTF8, #PB_Unicode or 0
      WriteLong(fid, slen)        ; datalength    (in byte)
      WriteLong(fid, scrc)        ; data CRC32
      WriteData(fid, *src, slen)  ; data
    Else                            ; erfolgreich komprimiert
      dcrc = CRC32Fingerprint(*dst, dlen)   ; Prfsumme der gepackten Daten berechnen
      WriteLong(fid, 18 + dlen)   ; blocksize
      WriteByte(fid, 1)           ; packed
      WriteByte(fid, flags)       ; #PB_Ascii, #PB_UTF8, #PB_Unicode or 0
      WriteLong(fid, slen)        ; datalength    (in byte)
      WriteLong(fid, dlen)        ; packedlength  (in byte)
      WriteLong(fid, scrc)        ; data CRC32
      WriteLong(fid, dcrc)        ; packeddata crc32
      WriteData(fid, *dst, dlen)  ; data
    EndIf
  
    If *dst : FreeMemory(*dst) : EndIf
    
    ProcedureReturn #True 
  EndIf
  
  ProcedureReturn #False
EndProcedure
Procedure.b ____FILE_WriteString(fid.i, str.s, format.b = #PB_Unicode, complevel.i = #____ZIP_COMP_MAX) ; schreibt einen String in die Datei und komprimiert diesen ggf
  Protected *buf, len.i, res.b, blen.i
  
  res = #False
  If fid And str
    ; Speicher anfordern fr den String im gewnschten Format und anschlieend inkl. Chr(0)-Abschluss in die Datei schreiben, falls gewnscht gepackt
    len  = Len(str)
    blen = StringByteLength(str, format) + 2
    *buf = AllocateMemory(blen)
    If *buf
      PokeS(*buf, str, len, format)
      PokeW(*buf + blen - 2, 0)
      res = ____FILE_WriteBlock(fid, *buf, blen, format, complevel)
      FreeMemory(*buf)  
    EndIf
  EndIf
  
  ProcedureReturn res
EndProcedure
Procedure.b ____FILE_WriteStringHdr(fid.i, hdr.s, str.s, format.b = #PB_Unicode, complevel.i = #____ZIP_COMP_MAX)
  If fid And hdr 
    If str
      If ____FILE_WriteHeader(fid, hdr)
        If ____FILE_WriteString(fid, str, format, complevel)
          ProcedureReturn #True
        EndIf
      EndIf
    Else
      ProcedureReturn #True      
    EndIf
  EndIf
  ProcedureReturn #False
EndProcedure
Procedure.i ____FILE_CreateFile(filename.s, hdr.s)
  Protected fid.i
  
  If filename And hdr
    fid = CreateFile(#PB_Any, filename)
    If IsFile(fid)      
      If ____FILE_WriteHeader(fid, hdr)
        ProcedureReturn fid        
      EndIf
      CloseFile(fid)
    EndIf
  EndIf
  
  ProcedureReturn #Null
EndProcedure
Procedure.b ____FILE_FinishFile(fid.i)
  If IsFile(fid)
    If ____FILE_WriteHeader(fid, "EOF.")
      CloseFile(fid)
      
      ProcedureReturn #True
    EndIf
  EndIf
  
  ProcedureReturn #False
EndProcedure
Procedure.i ____FILE_ReadBlock(fid.i, *len, *flags)
  Protected *src, *dst, slen.i, dlen.i, packed.b, pos.i, sz.i, len.i, scrc.l, dcrc.l, crc.l, flags.b
  
  *dst = #Null
  If *len And *flags And fid
    PokeL(*len, 0)
    If Not Eof(fid)
      sz     = ReadLong(fid)
      pos    = Loc(fid)
      packed = ReadByte(fid)
      flags  = ReadByte(fid) : PokeB(*flags, flags)
      *src   = #Null
      len    = 0
      If packed
        dlen = ReadLong(fid)
        slen = ReadLong(fid)
        dcrc = ReadLong(fid)
        scrc = ReadLong(fid)
        *src = AllocateMemory(slen)
        *dst = AllocateMemory(dlen)
        If *src And *dst
          If ReadData(fid, *src, slen) = slen
            crc = CRC32Fingerprint(*src, slen)
            If crc = scrc
              *dst = ____ZIP_UnpackMemory(*src, slen, @len)
            EndIf
          EndIf
        EndIf
      Else
        dlen = ReadLong(fid)
        dcrc = ReadLong(fid)
        *dst = AllocateMemory(dlen)
        If *dst
          If ReadData(fid, *dst, dlen) = dlen
            len = dlen
          EndIf
        EndIf
      EndIf
      
      FileSeek(fid, pos + sz)
      
      If len = dlen
        crc = CRC32Fingerprint(*dst, dlen)
        If crc = dcrc
          PokeI(*len, len)
        Else
          FreeMemory(*dst)
          *dst = #Null
        EndIf
      Else
        FreeMemory(*dst)
        *dst = #Null
      EndIf   
      If *src : FreeMemory(*src) : EndIf
    EndIf
  EndIf
  
  If Not *dst : PokeI(*len, 0) : EndIf
  ProcedureReturn *dst
EndProcedure
Procedure.s ____FILE_ReadHeader(fid.i)
  Protected hdr.s
  
  hdr = ""
  If fid
    If Not Eof(fid)
      hdr = Chr(ReadAsciiCharacter(fid)) + Chr(ReadAsciiCharacter(fid)) + Chr(ReadAsciiCharacter(fid)) + Chr(ReadAsciiCharacter(fid))
    EndIf
  EndIf
  
  ProcedureReturn hdr
EndProcedure
Procedure.l ____FILE_ReadBlocksize(fid.i)
  Protected blocksize.l
  
  blocksize = -1
  If fid
    If Not Eof(fid)
      blocksize = ReadLong(fid)
    EndIf
  EndIf
  
  ProcedureReturn blocksize
EndProcedure
Procedure.s ____FILE_ReadString(fid.i)
  Protected len.i, res.s, *buf, format.b
  
  *buf = ____FILE_ReadBlock(fid, @len, @format)
  If *buf And len
    res = PeekS(*buf, -1, format)
    FreeMemory(*buf)
  Else
    res = ""
  EndIf
  
  ProcedureReturn res
EndProcedure
Procedure.b ____FILE_SkipBlock(fid.i)
  Protected sz.i, pos.i
  
  If fid
    sz  = ReadLong(fid)
    If sz > 0
      pos = Loc(fid)
      FileSeek(fid, pos + sz)
    EndIf
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure
Procedure.i ____FILE_ReadFile(filename.s, hdr.s)
  Protected fid.i
  
  If filename And hdr
    fid = ReadFile(#PB_Any, filename)
    If IsFile(fid)
      If ____FILE_ReadHeader(fid) = hdr
        ProcedureReturn fid
      EndIf
      CloseFile(fid)
    EndIf
  EndIf
  
  ProcedureReturn #Null
EndProcedure
Procedure   ____FILE_CloseFile(fid.i)
  If fid
    If IsFile(fid)
      CloseFile(fid)    
    EndIf
  EndIf
EndProcedure
;}
;{ public functions
;** awpb_file_writequadhdr
;* Schreibt einen Header mit Lngenfeld und einen QuadWert (8 Bytes) in die Datei._
;* Rckgabe ist #True bei erfolg, sonst #False
;** .fid.i: FileID
;** .hdr.s: Header; max 4 Zeichen werden benutzt
;** .value.q: Quad-Value
Procedure.b AWPB_FILE_WriteQuadHdr(fid.i, hdr.s, value.q)
  If fid And hdr
    If ____FILE_WriteHeader(fid, hdr)
      WriteLong(fid, 8)
      WriteQuad(fid, value)
      ProcedureReturn #True
    EndIf
  EndIf
  ProcedureReturn #False
EndProcedure
;** awpb_file_writelonghdr
;* Schreibt einen Header mit Lngenfeld und einen LongWert (4 Bytes) in die Datei._
;* Rckgabe ist #True bei erfolg, sonst #False
;** .fid.i: FileID
;** .hdr.s: Header; max 4 Zeichen werden benutzt
;** .value.l: Long-Value
Procedure.b AWPB_FILE_WriteLongHdr(fid.i, hdr.s, value.l)
  If fid And hdr
    If ____FILE_WriteHeader(fid, hdr)
      WriteLong(fid, 4)
      WriteLong(fid, value)
      ProcedureReturn #True
    EndIf
  EndIf
  ProcedureReturn #False
EndProcedure
;** awpb_file_writewordhdr
;* Schreibt einen Header mit Lngenfeld und einen WordWert (2 Bytes) in die Datei._
;* Rckgabe ist #True bei erfolg, sonst #False
;** .fid.i: FileID
;** .hdr.s: Header; max 4 Zeichen werden benutzt
;** .value.w: Word-Value
Procedure.b AWPB_FILE_WriteWordHdr(fid.i, hdr.s, value.w)
  If fid And hdr
    If ____FILE_WriteHeader(fid, hdr)
      WriteLong(fid, 2)
      WriteWord(fid, value)
      ProcedureReturn #True
    EndIf
  EndIf
  ProcedureReturn #False
EndProcedure
;** awpb_file_writebytehdr
;* Schreibt einen Header mit Lngenfeld und einen ByteWert (1 Byte) in die Datei._
;* Rckgabe ist #True bei erfolg, sonst #False
;** .fid.i: FileID
;** .hdr.s: Header; max 4 Zeichen werden benutzt
;** .value.b: Byte-Value
Procedure.b AWPB_FILE_WriteByteHdr(fid.i, hdr.s, value.b)
  If fid And hdr
    If ____FILE_WriteHeader(fid, hdr)
      WriteLong(fid, 1)
      WriteByte(fid, value)
      ProcedureReturn #True
    EndIf
  EndIf
  ProcedureReturn #False
EndProcedure
;** awpb_file_writestringhdr
;* Schreibt einen Header mit Lngenfeld und einen String in die Datei._ 
;* Das Format (ASCII/UTF8/UniCode) kann angegeben werden, genauso wie eine Kompressionsstufe. Falls keine Kompression mglich ist, wird der String unkomprimiert geschrieben._
;* Rckgabe ist #True bei erfolg, sonst #False
;** .fid.i: FileID
;** .hdr.s: Header; max 4 Zeichen werden benutzt
;** .str.s; String, der geschrieben werden soll
;** .format.b: Stringformat, einer folgender Werte:_
;* <ul><li>#PB_ASCII</li><li>#PB_UTF8</li><li>#PB_UniCode (default)</li></ul>
;** .complevel.i: Kompressionsstrke; einer der folgenden Werte:_
;* <ul><li>#AWPB_ZIP_COMP_0</li><li>#AWPB_ZIP_COMP_1</li><li>#AWPB_ZIP_COMP_2</li><li>#AWPB_ZIP_COMP_3</li><li>#AWPB_ZIP_COMP_4</li>
;* <li>#AWPB_ZIP_COMP_5</li><li>#AWPB_ZIP_COMP_6</li><li>#AWPB_ZIP_COMP_7</li><li>#AWPB_ZIP_COMP_8</li><li>#AWPB_ZIP_COMP_9</li>
;* <li>#AWPB_ZIP_COMP_STORE</li><li>#AWPB_ZIP_COMP_NONE</li><li>#AWPB_ZIP_COMP_FAST</li><li>#AWPB_ZIP_COMP_NORMAL</li><li>#AWPB_ZIP_COMP_MAX (default)</li>
Procedure.b AWPB_FILE_WriteStringHdr(fid.i, hdr.s, str.s, format.b = #PB_Unicode, complevel.i = #AWPB_ZIP_COMP_MAX)
  ProcedureReturn ____FILE_WriteStringHdr(fid, hdr, str, format, complevel)
EndProcedure
;** awpb_file_writebinaryhdr
;* Schreibt einen Header mit Lngenfeld und einen String in die Datei._ 
;* Falls keine Kompression mglich ist, wird der Binrblock unkomprimiert geschrieben._
;* Rckgabe ist #True bei erfolg, sonst #False.
;** .fid.i: FileID
;** .hdr.s: Header; max 4 Zeichen werden benutzt
;** .*binblock: Zeiger auf Binrdaten
;** .binlen.i: Lnge der Daten
;** .complevel.i: Kompressionsstrke; einer der folgenden Werte:_
;* <ul><li>#AWPB_ZIP_COMP_0</li><li>#AWPB_ZIP_COMP_1</li><li>#AWPB_ZIP_COMP_2</li><li>#AWPB_ZIP_COMP_3</li><li>#AWPB_ZIP_COMP_4</li>
;* <li>#AWPB_ZIP_COMP_5</li><li>#AWPB_ZIP_COMP_6</li><li>#AWPB_ZIP_COMP_7</li><li>#AWPB_ZIP_COMP_8</li><li>#AWPB_ZIP_COMP_9</li>
;* <li>#AWPB_ZIP_COMP_STORE</li><li>#AWPB_ZIP_COMP_NONE</li><li>#AWPB_ZIP_COMP_FAST</li><li>#AWPB_ZIP_COMP_NORMAL</li><li>#AWPB_ZIP_COMP_MAX (default)</li>
Procedure.b AWPB_FILE_WriteBinaryHdr(fid.i, hdr.s, *binblock, binlen.i, complevel.i = #AWPB_ZIP_COMP_MAX)
  If ____FILE_WriteHeader(fid, hdr)
    If ____FILE_WriteBlock(fid, *binblock, binlen, 0, complevel)
      ProcedureReturn #True
    EndIf
  EndIf
  ProcedureReturn #False
EndProcedure
;** awpb_file_createfile
;* Erzeugt eine Datei mit Header._
;* Bei einem Fehler wird #Null zurckgegeben, ansonsten wird die FileID zurckgegeben
;** .filename.s: Dateiname inkl Pfad
;** .hdr.s: allgemeiner Header der Datei; max 4 Zeichen werden benutzt
Procedure.i AWPB_FILE_CreateFile(filename.s, hdr.s)
  ProcedureReturn ____FILE_CreateFile(filename, hdr)
EndProcedure
;** awpb_file_readfile
;* ffnet eine Datei. Falls der Header nicht passt oder ein Fehler auftrag wird #False zurckgegeben; ansonsten #True.
;** .filename.s: Dateiname inkl Pfad
;** .hdr.s: allgemeiner Header der Datei; max 4 Zeichen werden benutzt
Procedure.i AWPB_FILE_ReadFile(filename.s, hdr.s)
  ProcedureReturn ____FILE_ReadFile(filename, hdr)
EndProcedure
;** awpb_file_finishfile
;* Schreibt einen abschlieenden Header und schliet die Datei._
;* Bei einem Fehler wird #False zurckgegeben, ansonsten #True.
;** .fid.i: FileID
Procedure.b AWPB_FILE_FinishFile(fid.i)
  ProcedureReturn ____FILE_FinishFile(fid)
EndProcedure
;** awpb_file_closefile
;* Schliet die Datei.
;** .fid.i: FileID
Procedure   AWPB_FILE_CloseFile(fid.i)
  ProcedureReturn ____FILE_CloseFile(fid)
EndProcedure
;** awpb_file_readheader
;* Liest den nchsten Header aus der Datei (4 Bytes) und gibt ihn als String zurck._
;* Tritt ein Fehler auf, wird ein leerer String zurckgegeben.
;** .fid.i: FileID
Procedure.s AWPB_FILE_ReadHeader(fid.i)
  ProcedureReturn ____FILE_ReadHeader(fid)
EndProcedure
;** awpb_file_readblocksize
;* Liest die Gre des folgenden Blocks aus._
;* ACHTUNG: Nicht aufrufen, wenn danach ReadBlock(), ReadString() oder SkipBlock() aufgerufen wird!_
;* ReadBlockSize() darf nur verwendet werden, wenn die folgenden Daten mittels der PureBasic-Funktionen gelesen werden sollen. Dazu gehren:_
;*<ul><lu>ReadByte()</li><lu>ReadWord()</li><lu>ReadLong()</li><lu>ReadQuad()</li></ul>
;** .fid.i: FileID
Procedure.l AWPB_FILE_ReadBlocksize(fid.i)
  ProcedureReturn ____FILE_ReadBlocksize(fid)
EndProcedure
;** awpb_file_readblock
;* Liest den nchsten Binrblock aus der Datei, dekomprimiert ihn ggf und gibt die Adresse auf den Speicher zurck._
;* Die Lnge wird in len.i gespeichert, dessen Adresse bergeben werden muss. Im Fehlerfall wird #Null zurckgegeben._
;* Die zurckgegebene Adresse muss mit AWPB_FILE_FreeBlock() freigegeben werden.
;** .fid.i: FileID
Procedure.i AWPB_FILE_ReadBlock(fid.i, *len)
  Protected flags.i
  
  ProcedureReturn ____FILE_ReadBlock(fid, *len, @flags)
EndProcedure
;** awpb_file_freeblock
;* Gibt den Speicher des mit AWPB_FILE_ReadBlock() gelesenen Blockes frei.
;** .*blockadr: Adresse des gelesenen Blocks
Procedure.i AWPB_FILE_FreeBlock(*blockadr)
  If *blockadr
    FreeMemory(*blockadr)
  EndIf
EndProcedure
;** awpb_file_readstring
;* Liest den nchsten String aus der Datei und gibt ihn zurck._
;* Tritt ein Fehler auf, wird ein leerer String zurckgegeben.
;** .fid.i: FileID
Procedure.s AWPB_FILE_ReadString(fid.i)
  ProcedureReturn ____FILE_ReadString(fid)
EndProcedure
;** awpb_file_skipblock
;* berspringt den aktuellen Block.
;* Tritt ein Fehler auf, wird #False zurckgegeben, ansonsten #True.
;** .fid.i: FileID
Procedure.b AWPB_FILE_SkipBlock(fid.i)
  ProcedureReturn ____FILE_SkipBlock(fid)
EndProcedure
;}
 
; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 372
; Folding = AAAAA+
; EnableUnicode
; EnableThread
; EnableXP
; CPU = 1