; *****************************
; ** angelZware PB Tools Lib **
; ** ----------------------- **
; ** awsmtp.pbi              **
; ** V0.3 (2010/09/09)       **
; ** by Ronny Krger         **
; *****************************
; ** Licence: LGPL V3        **
; *****************************

XIncludeFile "awnetwork.pbi"
XIncludeFile "awsupport.pbi"
XIncludeFile "awmessage.pbi"

CompilerIf Defined(Enable_OpenSSL, #PB_Constant)
  XIncludeFile "awopenssl.pbi"
CompilerEndIf

;**
;* awsmtp.pbi_
;* Version 0.3_
;* _
;* SMTP-Client fr PureBasic 4.50+_
;* Lizenz: LGPL V3_
;* _
;* Systemanforderungen:_
;* <table style="text-align: left;" border="0" cellpadding="2" cellspacing="0"><tbody><tr>
;* <td style="vertical-align: top;">Windows</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Linux</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Mac</td><td style="vertical-align: top;">[ ]</td></tr><tr>
;* <td style="vertical-align: top;">x86</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">x64</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">PPC</td><td style="vertical-align: top;">[ ]</td></tr><tr>
;* <td style="vertical-align: top;">ASCII</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Unicode</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;"></td><td style="vertical-align: top;"></td></tr></tbody></table>_
;* Wichtig: Vor der Benutzung muss einmalig InitNetwork() aufgerufen werden!

;{ private functions
;{ Structures
Structure ____SMTP_Conn Extends ____NW_NetworkConn
  Caps.s
  ;- offen fr Erweiterungen
EndStructure
;}
;{ Constants
#____SMTP_Connect_PLAIN  = $0001
#____SMTP_Connect_LOGIN  = $0002
#____SMTP_Connect_AUTO   = $0003
;}
;{ functions
Procedure.s ____SMTP_FindCapa(*smtp.____SMTP_Conn, capa.s)             ; testen, ob der Server ein Feature untersttzt (suche im CAPS-String)
  Protected res.s, s.l, e.l
  
  res = ""
  s   = FindString(UCase(*smtp\caps), UCase(capa), 1)
  If s
    e   = FindString(*smtp\caps, #____NW_CRLF, s)
    If e = 0 
      e = Len(*smtp\caps)  
    EndIf 
    res = Trim(Mid(*smtp\caps, s, e - s))
  EndIf
  
  ProcedureReturn res
EndProcedure

Procedure.i ____AllocateSMTP(TimeOut.i = 15000, Retries.i = 5)           ; SMTP-Datenstuktur anfordern
  Protected *smtp.____SMTP_Conn
  
  *smtp = AllocateMemory(SizeOf(____SMTP_Conn))                         ; Speicher alloziieren
  If *smtp
    InitializeStructure(*smtp, ____SMTP_Conn)                           ; Datenstruktur initialisieren
    *smtp\TimeOut      = TimeOut                                        ; TimeOut (Standard: 15 Sekunden)
    *smtp\Retries      = Retries                                        ; Retries im Fehlerfall (standard: 5)
    *smtp\CallbackFunc = #Null
    *smtp\Buffer       = AllocateMemory(#____NW_BUF_SIZE + 2)           ; Sende- und Empfangspuffen anfordern; max 65000 Bytes
    If Not *smtp\Buffer
      FreeMemory(*smtp)
      *smtp = #Null
    EndIf
  EndIf
  
  ProcedureReturn *smtp
EndProcedure
Procedure   ____FreeSMTP(*smtp.____SMTP_Conn)                            ; SMTP-Datenstruktur freigeben
  If *smtp
    If *smtp\Buffer
      FreeMemory(*smtp\Buffer)                                         ; Sende-/Empfangspuffer freigeben
    EndIf
    FreeMemory(*smtp)
  EndIf
EndProcedure

Procedure.b ____SMTPRset(*smtp.____SMTP_Conn)                            ; SMTP-Befehl RSET senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *smtp\Retries
  Repeat
    res = ____NW_Execute(*smtp, "RSET", *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 250:   res = #True
        Default:    res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPNoOp(*smtp.____SMTP_Conn)                            ; SMTP-Befehl NOOP senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *smtp\Retries
  Repeat
    res = ____NW_Execute(*smtp, "NOOP", *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 250:   res = #True
        Default:    res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPEtrn(*smtp.____SMTP_Conn, Value.s)                   ; SMTP-Befehl ETRN senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *smtp\Retries
  Repeat
    res = ____NW_Execute(*smtp, "ETRN " + Value, *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 250 To 259:    res = #True
        Default:            res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPVerify(*smtp.____SMTP_Conn, Value.s)                 ; SMTP-Befehl VRFY senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *smtp\Retries
  Repeat
    res = ____NW_Execute(*smtp, "VRFY " + Value, *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 250 To 259:    res = #True
        Default:            res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPMailFrom(*smtp.____SMTP_Conn, Value.s)               ; SMTP-Befehl MAIL FROM senden und Ergebnis auswerten
  Protected res.b, retries.i
  retries = *smtp\Retries
  Repeat
    res     = ____NW_Execute(*smtp, "MAIL FROM:<" + Value + ">", *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 250:   res = #True
        Default:    res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPMailTo(*smtp.____SMTP_Conn, Value.s)                 ; SMTP-Befehl RCPT TO senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *smtp\Retries
  Repeat  
    res = ____NW_Execute(*smtp, "RCPT TO:<" + Value + ">", *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 250:   res = #True
        Default:    res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPEhlo(*smtp.____SMTP_Conn, host.s)                    ; SMTP-Befehl EHLO senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *smtp\Retries
  Repeat  
    res = ____NW_Execute(*smtp, "EHLO " + Host, *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 250 To 259:    res = #True
        Default:            res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPHelo(*smtp.____SMTP_Conn, host.s)                    ; SMTP-Befehl HELO senden und Ergebnis auswerten
  Protected res.b, retries.i 
  
  retries = *smtp\Retries
  Repeat  
    res = ____NW_Execute(*smtp, "HELO " + Host, *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 250 To 259:    res = #True
        Default:            res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPAuthPlain(*smtp.____SMTP_Conn)                       ; SMTP-Befehl AUTH PLAIN senden und Ergebnis auswerten
  Protected res.b, *aup, aupl.i, ul.i, pl.i, userpassb64.s, retries.i
  
  res  = #False
  ul   = Len(*smtp\User)
  pl   = Len(*smtp\Pass)
  aupl = ul + ul + pl + 2
  *aup = AllocateMemory(aupl + 2)
  If *aup
    PokeS(*aup,               *smtp\User, ul, #PB_Ascii)
    PokeA(*aup + ul,          0)
    PokeS(*aup + ul + 1,      *smtp\User, ul, #PB_Ascii)
    PokeA(*aup + ul + ul + 1, 0)
    PokeS(*aup + ul + ul + 2, *smtp\Pass, pl, #PB_Ascii)    
    userpassb64 = ____MSG_EncodeBase64BinToStr(*aup, aupl, #PB_Any)
    FreeMemory(*aup)
    retries = *smtp\Retries
    Repeat
      res = ____NW_Execute(*smtp, "AUTH PLAIN " + userpassb64, *smtp\Retries)
      If res
        Select *smtp\ResultCode
          Case 235:   res = #True
          Case 535:   res = #False : retries = 0
          Default:    res = #False : retries - 1
        EndSelect
      EndIf
    Until res Or (retries <= 0)
  EndIf
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPAuthLogin(*smtp.____SMTP_Conn)                       ; SMTP-Befehl AUTH LOGIN senden und Ergebnis auswerten
  Protected res.b, userb64.s, passb64.s, retries.i
  
  userb64 = ____SUP_TrimWS(____MSG_EncodeBase64StrToStr(*smtp\User))
  passb64 = ____SUP_TrimWS(____MSG_EncodeBase64StrToStr(*smtp\Pass))
  retries = *smtp\Retries
  Repeat
    res = ____NW_Execute(*smtp, "AUTH LOGIN", *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 334:   res = #True
        Default:    res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  If res
    retries = 1; *smtp\Retries
    Repeat
      res = ____NW_Execute(*smtp, userb64, *smtp\Retries)
      If res
        Select *smtp\ResultCode
          Case 334:   res = #True
          Case 535:   res = #False : retries = 0
          Default:    res = #False : retries - 1
        EndSelect
      EndIf
    Until res Or (retries <= 0)
  EndIf
  
  If res    
    retries = 1 ;*smtp\Retries
    Repeat
      res = ____NW_Execute(*smtp, passb64, *smtp\Retries)
      If res
        Select *smtp\ResultCode
          Case 235:   res = #True
          Case 535:   res = #False : retries = 0
          Default:    res = #False : retries - 1
        EndSelect
      EndIf
    Until res Or (retries <= 0)
  EndIf
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPQuit(*smtp.____SMTP_Conn)                            ; SMTP-Befehl QUIT senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *smtp\Retries
  Repeat
    res = ____NW_Execute(*smtp, "QUIT", *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 221:   res = #True
        Default:    res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPStartMailData(*smtp.____SMTP_Conn)                   ; SMTP-Befehl DATA senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *smtp\Retries
  Repeat  
    res = ____NW_Execute(*smtp, "DATA", *smtp\Retries)
    If res
      Select *smtp\ResultCode
        Case 354:   res = #True
        Default:    res = #False : retries - 1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPMailData(*smtp.____SMTP_Conn, Value.s)               ; Maildaten senden (als String bergeben)
  Protected trimmed.s
  
  ;- ToDo: . am Zeilenanfang besonders behandeln (verdoppeln)
  
  trimmed = ____SUP_TrimWS(Value)         ; Maildaten vorher trimmen (v.a. am Ende wichtig)
  
  ProcedureReturn ____NW_SendNetworkStr(*smtp, trimmed, #False)
EndProcedure
Procedure.b ____SMTPMailDataBin(*smtp.____SMTP_Conn, *buf, bufsize.i)    ; Maildaten senden (direkt ab Speicher)
  Protected len.i, *trimmed
  
  ;- ToDo: . am Zeilenanfang besonders behandeln (verdoppeln)
  
  *trimmed = ____SUP_TrimWSBin(*buf, bufsize, @len) 

  ProcedureReturn ____NW_SendNetworkBin(*smtp, *trimmed, len)
EndProcedure
Procedure.b ____SMTPFinishMailData(*smtp.____SMTP_Conn)                  ; SMTP-Befehl CRLF.CRLF senden und Ergebnis auswerten
  Protected res.b
  
  res = ____NW_Execute(*smtp, #____NW_CRLF + ".", *smtp\Retries)
  If res
    Select *smtp\ResultCode
      Case 250:   res = #True
      Default:    res = #False
    EndSelect
  EndIf
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPStartTLS(*smtp.____SMTP_Conn)
  Protected res.b
  
  res     = #False
  
  CompilerIf Defined(Enable_OpenSSL, #PB_Constant)
    retries = *smtp\Retries
    Repeat
      res = ____NW_Execute(*smtp, "STARTTLS", *smtp\Retries)
      If res
        Select *smtp\ResultCode
          Case 220:   res = #True
          Default:    res = #False : retries - 1
        EndSelect
      EndIf
    Until res Or (retries <= 0)
    
    If res
      res = ____SSL_DoConnect(*smtp)
    EndIf
  CompilerEndIf
  
  ProcedureReturn res
EndProcedure

Procedure.b ____SMTPConnect(*smtp.____SMTP_Conn, mode.i, autotls.b) ; mit SMTP-Server verbinden
  Protected c.s, auths.s, tmp.i, ok.b, res.b

  res          = #False
  *smtp\caps   = "" 
  *smtp\ConnID = OpenNetworkConnection(*smtp\Host, *smtp\Port)
  If *smtp\ConnID
    If ____NW_ReadResult(*smtp) = 220
      res = ____SMTPEhlo(*smtp, Hostname())                               ; EHLO senden, falls nicht erfolgreich, das Anmelden mit
      If Not res : res = ____SMTPHelo(*smtp, Hostname()) : EndIf         ; HELO erneut versuchen
      If res
        *smtp\caps = *smtp\ResultString                                   ; Features-String des Servers (nur bei EHLO verfgbar) speichern        
        CompilerIf Defined(Enable_OpenSSL, #PB_Constant)
          If autotls And FindString(auths, "STARTTLS", 1)                 ; TLS starten; OpenSSL erforderlich [hierzu #Enable_OpenSSL VOR include der datei setzen]
            *smtp\caps = ""
            res        = ____SMTPStartTLS(*smtp)
            If res
              res = ____SMTPEhlo(*smtp, Hostname())
              If res
                *smtp\caps = *smtp\ResultString
              EndIf
            EndIf
          EndIf
        CompilerEndIf
      EndIf
      If res
        res   = #False
        auths = UCase(____SMTP_FindCapa(*smtp, "AUTH "))
        If Not auths : auths = UCase(____SMTP_FindCapa(*smtp, "AUTH=")) : EndIf
        If auths
          If (mode & #____SMTP_Connect_PLAIN) And FindString(auths, "PLAIN", 1)   ; falls noch nicht eingeloggt, nchste unsicherere methode versuchen
            res = ____SMTPAuthPlain(*smtp)
          EndIf
          If Not res And (mode & #____SMTP_Connect_LOGIN) And FindString(auths, "LOGIN", 1)   ; falls noch nicht eingeloggt, nchste unsicherere methode versuchen
            res = ____SMTPAuthLogin(*smtp)
          EndIf
        Else
          If *smtp\caps
            res = #True     ; fr z.B. T-Online ntig, wo ber Anschluss authentifiziert wird
          EndIf
        EndIf
      EndIf  
    EndIf
    If Not res      ; nicht eingeloggt? Verbindung komplett trennen
      CloseNetworkConnection(*smtp\ConnID)
      *smtp\ConnID = #Null
    EndIf
  EndIf
  
  ProcedureReturn res
EndProcedure
Procedure.b ____SMTPDisConnect(*smtp.____SMTP_Conn)                      ; vom SMTP-Server trennen
  If ____SMTPQuit(*smtp)
    CloseNetworkConnection(*smtp\ConnID)
    *smtp\ConnID = #Null
    
    ProcedureReturn #True
  EndIf
  
  ProcedureReturn #False
EndProcedure
Procedure.b ____SMTPSendMail(*smtp.____SMTP_Conn, *msg.____MSG_Message)  ; Mail senden; bergeben werden muss das Message-Objekt
  Protected an.s, cc.s, bcc.s
  

  ;- todo: *msg muss dekodiert sein (flag einfgen); wenn nicht, dann jetzt zumindest den header dekodieren
  
  If (*smtp <> 0) And (*msg <> 0)
    If *msg\src
      If *msg\decoded = #False : ____MSG_AnalyzeMessage(*msg) : EndIf
      If *msg\decoded
        If ____SMTPMailFrom(*smtp, ____MSG_GetFrom(*msg))
          an  = ____MSG_GetTo(*msg)
          cc  = ____MSG_GetCC(*msg)
          bcc = ____MSG_GetBCC(*msg)
          
          If Len(cc) > 0
            If Len(an) > 0
              an + ", " + cc
            Else
              an = cc
            EndIf
          EndIf
          If Len(bcc) > 0
            If Len(an) > 0
              an + ", " + bcc
            Else
              an = bcc
            EndIf
          EndIf 
          
          ;- TODO: an darf nur die adressen enthalten; ohne irgendwelche Namen oder klammern o.. drumherum
          If ____SMTPMailTo(*smtp, an)
            If ____SMTPStartMailData(*smtp)
              If ____SMTPMailDataBin(*smtp, *msg\src, MemorySize(*msg\src) - 1)
                If ____SMTPFinishMailData(*smtp)
                  ProcedureReturn #True
                EndIf
              EndIf  
            EndIf
          EndIf
        EndIf
      EndIf
    EndIf
  EndIf  

  ProcedureReturn #False
EndProcedure
Procedure.s ____SMTPGetServerResult(*smtp.____SMTP_Conn)                 ; Resultstring zur Fehlerausgabe zurckgeben
  If *smtp
    ProcedureReturn *smtp\ResultString
  EndIf
  
  ProcedureReturn ""
EndProcedure
Procedure.i ____SMTPGetServerResultCode(*smtp.____SMTP_Conn)             ; Resultcode zur Fehlerausgabe zurckgeben
  If *smtp
    ProcedureReturn *smtp\ResultCode
  EndIf
  
  ProcedureReturn -1
EndProcedure
;}
;}
;{ public functions
;{ Constants
#AWPB_SMTP_Connect_PLAIN  = #____SMTP_Connect_PLAIN                       ;* AUTH PLAIN benutzen
#AWPB_SMTP_Connect_LOGIN  = #____SMTP_Connect_LOGIN                       ;* AUTH LOGIN benutzen
#AWPB_SMTP_Connect_AUTO   = #____SMTP_Connect_AUTO                        ;* Verbindungsmethode automatisch bestimmen
;}
;** awpb_smtp_allocatesmtp
;* Fordert Speicher fr ein SMTP-Objekt an._
;* Rckgabewert ist *SMTPObject. Ist der Rckgabewert #Null, wurde kein Objekt angelegt.
Procedure.i AWPB_SMTP_AllocateSMTP()
  ProcedureReturn ____AllocateSMTP()
EndProcedure
;** awpb_smtp_freesmtp
;* Gibt ein SMTP-Objekt frei.
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
Procedure   AWPB_SMTP_FreeSMTP(*SMTPObject)
  ____FreeSMTP(*SMTPObject)
EndProcedure

;** awpb_smtp_setserver
;* Setzt die Serverdaten des Objektes.
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
;** .HostName.s: Name des Servers
;** .Port.w: Portnummer
Procedure.b AWPB_SMTP_SetServer(*SMTPObject, HostName.s, Port.w)
  Protected *smtp.____SMTP_Conn
  
  *smtp = *SMTPObject
  If *smtp
    *smtp\Host = HostName
    *smtp\Port = Port
    
    ProcedureReturn #True
  EndIf
  
  ProcedureReturn #False
EndProcedure
;** awpb_smtp_setlogin
;* Setzt die Logindaten des Objektes.
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
;** .User.s: Benutzername
;** .Pass.s: Passwort
Procedure.b AWPB_SMTP_SetLogin(*SMTPObject, User.s, Pass.s)
  Protected *smtp.____SMTP_Conn
  
  *smtp = *SMTPObject
  If *smtp
    *smtp\User = User
    *smtp\Pass = Pass
    
    ProcedureReturn #True
  EndIf
  
  ProcedureReturn #False
EndProcedure

;** awpb_smtp_connect
;* Stellt eine Verbindung zum Server her. Vorher mssen mit AWPB_SMTP_SetServer() und AWPB_SMTP_SetLogin() die Logindaten gesetzt werden._
;* Rckgabewert ist bei Erfolg #True, im Fehlerfall #False
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
;** .mode: Verbindungsmodus; einer dieser Werte:_
;* <ul><li><b>#AWPB_SMTP_Connect_AUTO</b></li><li>#AWPB_SMTP_Connect_LOGIN</li><li>#AWPB_SMTP_Connect_PLAIN</li></ul>
Procedure.b AWPB_SMTP_Connect(*SMTPObject, mode.i = #AWPB_SMTP_Connect_AUTO)
  ProcedureReturn ____SMTPConnect(*SMTPObject, mode, #False)
EndProcedure
;** awpb_smtp_disconnect
;* Trennt die Verbindung mit dem Server_
;* Rckgabewert ist bei Erfolg #True, im Fehlerfall #False
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
Procedure.b AWPB_SMTP_DisConnect(*SMTPObject)
  ProcedureReturn ____SMTPDisConnect(*SMTPObject)
EndProcedure

;** awpb_smtp_rset
;* bermittelt dem Server den Befehl RSET._
;* Rckgabewert ist bei Erfolg #True, im Fehlerfall #False
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
Procedure.b AWPB_SMTP_RSET(*SMTPObject)
  ProcedureReturn ____SMTPRset(*SMTPObject)
EndProcedure
;** awpb_smtp_noop
;* bermittelt dem Server den Befehl NOOP._
;* Rckgabewert ist bei Erfolg #True, im Fehlerfall #False
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
Procedure.b AWPB_SMTP_NOOP(*SMTPObject)
  ProcedureReturn ____SMTPNoOp(*SMTPObject)
EndProcedure
;** awpb_smtp_sendmessage
;* Sendet das bergebene MessageObjekt. Alle relevanten Daten werden daraus entnommen.
;* Rckgabewert ist bei Erfolg #True, im Fehlerfall #False
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
;** .*MessageObject: Zeiger auf ein Message-Objekt
Procedure.b AWPB_SMTP_SendMessage(*SMTPObject, *MessageObject)
  ProcedureReturn ____SMTPSendMail(*SMTPObject, *MessageObject)
EndProcedure

;** awpb_smtp_getserverresult
;* Gibt den letzten vom Server erhaltenen String zurck, z.B. zur Ausgabe einer Fehlermeldung durch das Programm.
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
Procedure.s AWPB_SMTP_GetServerResult(*SMTPObject)
  ProcedureReturn ____SMTPGetServerResult(*SMTPObject)
EndProcedure
;** awpb_smtp_getserverresultcode
;* Gibt den letzten vom Server erhaltenen Fehlercode zurck, z.B. zur Ausgabe einer Fehlermeldung durch das Programm.
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
Procedure.i AWPB_SMTP_GetServerResultCode(*SMTPObject)
  ProcedureReturn ____SMTPGetServerResultCode(*SMTPObject)
EndProcedure

;** awpb_smtp_setcallback
;* Setzt eine Callback-Funktion, die an verschiedenen Stellen automatisch aufgerufen wird._ 
;* Die Callback-Funktion muss folgendem Muster entsprechen: <b>Procedure.b CallbackFunction(StatusText.s)</b>._
;* Der Prozedur wird ein String mit dem aktuellen Status bergeben. Ist der Rckgabewert der Funktion #False, wird die SMTP-Operation abgebrochen._
;* Wird als Zeiger #Null bergeben, wird die gespeicherte Prozedur entfernt.
;** .*SMTPObject: Zeiger auf ein SMTP-Objekt
;** .*CallbackFuncPointer: Zeiger auf eine Callbackfunktion oder #Null
Procedure   AWPB_SMTP_SetCallback(*SMTPObject, *CallbackFuncPointer = #Null)
  ____NW_SetCallbackFunc(*SMTPObject, *CallbackFuncPointer)
EndProcedure
;}
    
; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 269
; FirstLine = 88
; Folding = RAwAhAA-
; EnableThread
; EnableXP
; CPU = 1
; Compiler = PureBasic 4.50 (Windows - x64)